#    Kye - classic puzzle game
#    Copyright (C) 2005, 2006 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import pygtk
pygtk.require('2.0')
import gtk
from kye.canvas import KCanvas
from kye.input import KMoveInput
from kye.stbar import StatusBar
from kye.dialogs import GotoDialog, KyeHelpDialog, KyeAboutDialog

ui_string = """<ui>
 <menubar name='KyeMenuBar'>
  <menu action='FileMenu'>
   <menuitem action='Open' />
   <menuitem action='Quit' />
  </menu>
  <menu action='LevelMenu'>
   <menuitem action='Restart Level' />
   <menuitem action='Goto Level' />
  </menu>
  <menu action='ViewMenu'>
   <menuitem action='Tiny' />
   <menuitem action='Small' />
   <menuitem action='Medium' />
   <menuitem action='Large' />
  </menu>
  <menu action='HelpMenu'>
   <menuitem action='The Game' />
   <menuitem action='About Kye' />
  </menu>
 </menubar>
</ui>"""

class KFrame(gtk.Window):

    def delete_event(self, widget, event, data=None):
        # If you return FALSE in the "delete_event" signal handler,
        # GTK will emit the "destroy" signal. Returning TRUE means
        # you don't want the window to be destroyed.
        # This is useful for popping up 'are you sure you want to quit?'
        # type dialogs.
        return False

    # Another callback
    def destroy(self, widget, data=None):
        gtk.main_quit()

    def build_ui(self, restartcall):
        ag = gtk.ActionGroup('WindowActions')
	accel_group = gtk.AccelGroup()

	actions = [
		("FileMenu", None, "_File"),
		("Open", gtk.STOCK_OPEN, "_Open", "<control>O", "Open a set of levels", self.opendialog),
		("Quit", gtk.STOCK_QUIT, "_Quit", "<control>Q", "Quit the game", self.destroy),
		("LevelMenu", None, "_Level"),
		("Restart Level", None, "_Restart Level", "<control>R", "Restart this level", restartcall),
		("Goto Level", None, "_Goto Level", "<control>G", "Jump to a named level", self.startgoto),
		("ViewMenu", None, "_View"),
		("HelpMenu", None, "_Help"),
		("The Game", gtk.STOCK_HELP, "_The Game","<control>T", "Help for playing the game", self.helpdialog),
		("About Kye", gtk.STOCK_ABOUT, "About _Kye", "<control>K", "About Python Kye", self.aboutdialog),
	]
        radio_actions = [
		("Tiny", None, "_Tiny", "<control>1", "Set window size", 8),
		("Small", None, "_Small", "<control>2", "Set window size", 16),
		("Medium", None, "_Medium", "<control>3", "Set window size", 24),
		("Large", None, "_Large", "<control>4", "Set window size", 32)
        ]
        ag.add_actions(actions)
        ag.add_radio_actions(radio_actions, 16, self.settilesize)
        self.ui = gtk.UIManager()
        self.ui.insert_action_group(ag, 0)
        self.ui.add_ui_from_string(ui_string)
        self.add_accel_group(self.ui.get_accel_group())
	self.ignore_sizing = False

    def __init__(self, opencall = None, restartcall = None, gotocall = None):
	if gtk.pygtk_version[0] < 2 or gtk.pygtk_version[1] < 2:
	    raise EnvironmentError, "Needs at least pygtk-2.2.0"
        # create a new window
        gtk.Window.__init__(self,gtk.WINDOW_TOPLEVEL)
    
        # When the window is given the "delete_event" signal (this is given
        # by the window manager, usually by the "close" option, or on the
        # titlebar), we ask it to call the delete_event () function
        # as defined above. The data passed to the callback
        # function is NULL and is ignored in the callback function.
        self.connect("delete_event", self.delete_event)
    
        # Here we connect the "destroy" event to a signal handler.  
        # This event occurs when we call gtk_widget_destroy() on the window,
        # or if we return FALSE in the "delete_event" callback.
        self.connect("destroy", self.destroy)
    
        # Sets the border width of the window.
        self.set_title("Kye")
	self.set_resizable(False)
    
        self.moveinput = KMoveInput()
        # Creates a new canvas
        self.canvas = KCanvas(self.moveinput)
    
	# Main vbox
	self.main_vbox = gtk.VBox(False)
	self.main_vbox.set_border_width(1)

	# Build menu
        self.build_ui(restartcall=restartcall)
	self.menubar = self.ui.get_widget('/KyeMenuBar')
	self.main_vbox.pack_start(self.menubar, False, True, 0)
	self.menubar.show()

        # This packs the button into the window (a GTK container).
        self.main_vbox.pack_start(self.canvas, padding=0)
        self.canvas.show()

	# Status bar
	self.stbar = StatusBar(self.canvas.get_image("kye"))
	self.main_vbox.pack_start(self.stbar, padding=0)
	self.stbar.show()

	# Add and display the vbox
	self.add(self.main_vbox)
	self.main_vbox.show()

        # and the window
        self.set_icon(self.canvas.get_image("kye"))
        self.show()
	self.set_focus(self.canvas)
	self.gotocall = gotocall
	self.opencall = opencall

    def opendialog(self, w):
	filesel = gtk.FileChooserDialog("Open Kye Levels", buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        filter = gtk.FileFilter()
        filter.set_name("Kye Levels")
        filter.add_pattern("*.kye")
        filesel.set_filter(filter)
	response = filesel.run()
        if response == gtk.RESPONSE_OK:
            filename = filesel.get_filename()
            filesel.destroy()
            self.doopen(filename)
        else:
            filesel.destroy()

    def doopen(self,filename):
	self.opencall(filename)

    def startgoto(self,w):
	gd = GotoDialog(parent=self)
        r = gd.run()
	t = gd.entry.get_text()
        gd.destroy()
        if r == gtk.RESPONSE_ACCEPT:
            self.gotocall(t)

    def settilesize(self, ra, u):
	if not self.ignore_sizing:
	    self.canvas.settilesize(ra.get_current_value())

    def endleveldialog(self,nextlevel,endmsg):
	self.canvas.brelease(1,0,0)
	md = gtk.MessageDialog(self, gtk.DIALOG_MODAL, buttons=gtk.BUTTONS_OK, message_format=endmsg)
        if nextlevel != "":
            md.set_title("Level completed")
            nextlevelmsg = "Entering "+nextlevel
        else:
            md.set_title("All levels completed")
            nextlevelmsg = "Returning to first level"
	md.run()
        md.destroy()
        md = gtk.MessageDialog(self, gtk.DIALOG_MODAL, buttons=gtk.BUTTONS_OK, message_format=nextlevelmsg)
        md.run()
        md.destroy()
        self.gotocall(nextlevel)

    def helpdialog(self,a):
	hd = KyeHelpDialog(getimage=self.canvas.get_image, parent=self)
	hd.show()

    def aboutdialog(self,a):
	ad = KyeAboutDialog(kimg = self.canvas.get_image("kye"))
        ad.run()
        ad.destroy()

    def error_message(self,message):
	md = gtk.MessageDialog(parent=self,type=gtk.MESSAGE_ERROR,message_format=message,buttons=gtk.BUTTONS_OK)
	md.run()
	md.destroy()

    def main(self):
        # All PyGTK applications must have a gtk.main(). Control ends here
        # and waits for an event to occur (like a key press or mouse event).
        gtk.main()

